// Comparable.java
//
// ICS 22 / CSE 22 Fall 2012
// Code Example
//
// This is the Comparable interface.  Its job is to specify a "contract"
// that must be followed by any class that wants to consider its objects
// to be "comparable."  Comparable, in this context, means "comparable for
// the purposes of ordering," meaning that we can ask a comparable object
// how it compares to another, and get back one of the following answers:
//
//   - I'm smaller than the other one
//   - I'm larger than the other one
//   - I'm equal to the other one
//
// Furthermore, "comparable for the purposes of ordering" also implies
// that "smaller than", "larger than", and "equal to" are transitive
// (e.g., if a is smaller than b and b is smaller than c, then a is
// smaller than c).  We say that if some type of object is comparable
// in this fashion, that object has a "natural ordering."
//
// Now, the definition of "smaller," "larger," and "equal" are somewhat
// different for different kinds of objects.  In our example of dates, we
// defined "smaller" to mean "earlier on the calendar" and "larger" to mean
// "later on the calendar," which means that if we sorted dates, they would
// be sorted from earliest to latest.  We could similarly define an ordering
// for strings by considering "smaller" to be "earlier in an alphabetical
// order" and "larger" to be "later in an alphabetical order," which would
// allow us to sort strings alphabetically.
//
// The interface, though, doesn't specify these details.  It's totally
// abstract.  Java does not allow anything in an interface except:
//
//   - public abstract methods
//   - public constants
//
// Classes can agree to follow the rules set forth in this contract by
// including "implements Comparable" in their declaration.  (See MyDate.java
// for an example.)  This obligates the class to do two things:
//
//   - Include implementations of all of the methods in the interface.
//     (Alternatively, it can implement only some of them, but then the
//     class needs to be abstract, since there are details of its
//     implementation missing.)
//   - Follow the English part of the contract, too.  This means that the
//     method does the "right thing" according to the documentation in the
//     interface.
//
// Note that, as a practical matter, you wouldn't need to write this
// interface in Java, as there is already a Comparable interface built
// into the Java library, along with many existing methods in the Java
// library that make use of it to generically find minimums or maximums,
// do sorting, perform binary searches, and so on.
//
// I show this example here as a way to begin to get your head around how
// you write an interface, as well as how that corresponds to how you
// implement it in a class (as I've done in the CalendarDate class).


public interface Comparable
{
	// compareTo() compares this object to another object of the same type.
	// It returns one of the following results:
	//
	//   * a negative number, if this object is "smaller" than the other
	//   * a positive number, if this object is "larger" than the other
	//   * zero, if the objects are equivalent for purposes of ordering them
	//
	// If x refers to an object with which "this" object can't reasonably
	// be compared for the purposes of ordering (e.g., "this" is a
	// CalendarDate and "x" is a String), this method should throw a
	// ClassCastException.
	//
	// Note that, by convention, we generally don't put the word "abstract"
	// or "public" on methods in an interface, since all methods in an
	// interface are required to be public and abstract.
	int compareTo(Object x);
}
